#!/bin/bash

workdir="$(mktemp -d playlist-XXXXXX)"
ads_dir="../assets/ads/"
today="$(date +"%y/%m/%d")"
res_dir="radio/${today}"
current_player=""
job_started=0
mkdir -p "$res_dir"

function player(){
  if [[ -n "$current_player" ]];then
    # shellcheck disable=SC2145
    res="$(eval "playerctl --player=$current_player $@ || echo 'ERROR'")" 
  else
    # shellcheck disable=SC2145
    res="$(eval "playerctl $@ || echo 'ERROR'")"
  fi
  echo "$res"
}

function cleanup_and_exit() {
  if [[ "$job_started" = "1" ]];then
    rm -fv "current.mp3" "bg.png" "cover.jpg" "cover.png"
    find . -name '*.mp4' -exec basename {} \; | sort --numeric-sort | xargs -I {} echo "file '{}'" > concat
    ffmpeg -f concat -i concat -c copy conc.mp4 
    ffmpeg -i conc.mp4 -crf 23 "compiled-$((compiled_index + 1)).mp4"
    rm -fv "concat"
    popd || exit 
    find "$workdir" -maxdepth 1 -name "compiled*" -print0 | xargs -I '{}' mv -v '{}' "$res_dir"
  fi
  rm -rfv "$workdir"
}

function read_player(){
  echo "Which player should I use?"
  printf "Available players:\n%s\n" "$(playerctl -l | sed "s/^/* /")"
  # shellcheck disable=SC2162
  read -p '> ' current_player
}

function render_on_cpu(){
  background="$1"
  sound="$2"
  index="$3"
  target_adv="$4"
  ffmpeg -loop 1 -r 3 -i "$background" -i "$sound" -r 25 -i "${target_adv}" \
    -filter_complex "[0:v]scale=w=1280:h=720:force_original_aspect_ratio=2,crop=1280:720[img]; \
    [2:v]setpts=10*PTS[adv]; \
    [adv]loop[ladv]; \
    [img][ladv]overlay=enable='gt(mod(t, 40), 30)'[rv]" \
    -map "[rv]" \
    -map 1:a \
    -framerate 30 -g 60 \
    -c:v vp8 -pix_fmt yuv420p -movflags +faststart \
    -vb 4000k \
    -f webm \
    -c:a libopus \
    -err_detect ignore_err \
    -shortest -strict -2 -y \
    "${index}.webm"
}

function render_on_gpu(){
  background="$1"
  sound="$2"
  index="$3"
  target_adv="$4"
  ffmpeg -loop 1 -init_hw_device cuda=cuda:0 \
    -hwaccel nvdec \
    -r 15 -i "$background" -i "$sound" -stream_loop -1 -i "${target_adv}" \
    -filter_complex "[0:v]scale=w=1280:h=720:force_original_aspect_ratio=2,crop=1280:720[img]; \
    [2:v]loop=999,setpts=N/FRAME_RATE/TB[ladv]; \
    [img][ladv]overlay=enable='gt(mod(t, 40), 30)'[rv]" \
    -map "[rv]" \
    -map 1:a \
    -framerate 30 -g 60 \
    -c:v libx264 -preset ultrafast -crf 23 -pix_fmt yuv420p -movflags +faststart \
    -c:a aac \
    -b:v 400k \
    -b:a 384k \
    -err_detect ignore_err \
    -shortest -strict -2 -y \
    "${index}.mp4"
}

trap "cleanup_and_exit" EXIT

if [[ ! ($(playerctl -l | wc -l) -eq 1) ]];then
  read_player
else
  current_player="$(playerctl -l)"
fi

if [[ ! ($(player status) ) ]]; then
  echo "Can't find any player"
  exit 1
fi

if [[ $(player status) != "Playing" ]]; then
  player next
fi

if [[ -e "${res_dir}" ]];then
  compiled_index="$(find "$res_dir" -name "compiled*" | sort --numeric-sort | tail -n 1 | cut -d '.' -f1 | cut -d '-' -f2)"
else
  compiled_index="-1"
fi
index="0"

pushd "$workdir" || exit 1

job_started=1

while [[ "$(player status)" = "Playing" ]];do
  title="$(player metadata xesam:title)"
  artist="$(player metadata xesam:artist)"
  artUrl="$(player metadata mpris:artUrl)"
  url="$(player metadata xesam:url)"

  # Downloading song and album cover
  echo "Loading song"
  wget -q "${url}" -O "current.mp3"
  echo "loaded"

  echo "Loading album cover"
  wget -q "${artUrl}" -O "cover.jpg"
  echo "loaded"

  # Processing background
  music_bg test "cover.jpg" --output "cover.png"
  
  # Adding song name
  convert -background '#0008' -gravity center \
    -fill white -size 800x70 caption:"${artist} - ${title}" \
    "cover.png" +swap -gravity south -geometry +0+200 -composite "bg.png"
  
  rm -fv "cover.jpg" "cover.png"

  target_adv="$(find "$ads_dir" -type f | shuf -n 1)"
  echo "Rendering ${index}"
  if ! command -v nvcc &> /dev/null; then
    echo "Cuda was not found. Rendering on cpu."
    echo "Lowering fps..."
    render_on_cpu "bg.png" "current.mp3" "${index}" "${target_adv}"
  else
    echo "Cuda was found. Hardware acceleration enabled."
    render_on_gpu "bg.png" "current.mp3" "${index}" "${target_adv}"
  fi

  index=$((index + 1))
  if [[ "$(player next)" = "ERROR" ]];then
    break
  fi
done

player stop
